﻿using System;
using System.IO;
using System.IO.Ports;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace PDF417EAN
{
    public partial class MainForm : Form
    {
        private int oldKeyValue;
        FileStream fileStream;
        private StreamWriter streamWriter;

        //Для сканера ШК
        private delegate void SetTextCallback(string text);
        private static Control _barcodeControl;
        private static string _barcode;

        public MainForm()
        {
            InitializeComponent();

            cbComPortName.SelectedIndex = 0;

            string fname = "file";
            int i = 1;
            while (File.Exists(fname + ".txt") && new FileInfo(fname + ".txt").Length > 0)
            {
                fname = "file" + i;
                i++;
            }

            if (File.Exists(fname + ".txt")) File.Delete(fname + ".txt");

            fileStream = new FileStream(fname + ".txt", FileMode.CreateNew);
            streamWriter = new StreamWriter(fileStream);

            tbEAN.GotFocus += tbEAN_GotFocus;
            tbMarkCode.GotFocus += tbMarkCode_GotFocus;
        }

        private void tbMarkCode_GotFocus(object sender, EventArgs e)
        {
            _barcodeControl = tbMarkCode;
        }

        private void tbEAN_GotFocus(object sender, EventArgs e)
        {
            _barcodeControl = tbEAN;
        }

        private void SetText(string text)
        {
            if (_barcodeControl == null) { return; }
            if (_barcodeControl.InvokeRequired)
            {
                SetTextCallback d = SetText;
                _barcodeControl.Invoke(d, new object[] { text });
            }
            else
            {
                //text = text.Trim(); если тримить, то ToChar(3) отбрасывается
                if (text.StartsWith(new string(Convert.ToChar(2), 1)))
                {
                    text = text.Substring(1);
                }

                //ШК прочитался, парсим, убираем всякую хуйню
                if (text.EndsWith(new string(Convert.ToChar(3), 1)) || text.EndsWith(new string(Convert.ToChar(13), 1)))
                {
                    _barcode += text;
                    _barcodeControl.Text = _barcode;
                    _barcode = string.Empty;
                    if (_barcodeControl.Name.Equals(tbMarkCode.Name))
                    {
                        if (CheckMarkCode(tbMarkCode.Text.Trim()))
                        {
                            tbEAN.Focus();
                            tbEAN.SelectAll();
                        }
                        else
                        {
                            MessageBox.Show(this, "Неверный код марки", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            tbMarkCode.SelectAll();
                        }
                    }
                    else
                    {
                        if (string.IsNullOrEmpty(tbEAN.Text.Trim()) || CheckEAN(tbEAN.Text.Trim()))
                        {
                            if (!CheckMarkCode(tbMarkCode.Text.Trim()))
                            {
                                MessageBox.Show(this, "Неверный код марки", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                                tbMarkCode.SelectAll();
                                tbMarkCode.Focus();
                                return;
                            }

                            //Пишем в файл
                            streamWriter.WriteLine("{0} {1}", tbMarkCode.Text.Trim(), tbEAN.Text.Trim());
                            streamWriter.Flush();

                            tbMarkCode.Text = string.Empty;
                            tbEAN.Text = string.Empty;
                            tbMarkCode.Focus();
                        }
                        else
                        {
                            MessageBox.Show(this, "Неверный ШК", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            tbEAN.SelectAll();
                        }
                    }
                }
                else
                {
                    _barcode += text;
                }
            }
        }


        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            string sData = UtilsScannerBarcode.ReadExisting();
            if (!string.IsNullOrEmpty(sData))
            {
                SetText(sData);
            }
        }

        private bool CheckMarkCode(string code)
        {
            if (code == null || code.Length != 68) return false;

            const string pattern = @"^([A-Za-z0-9]+)$";
            var check = new Regex(pattern, RegexOptions.IgnoreCase);

            var valid = false;
            if (!string.IsNullOrEmpty(code))
            {
                valid = check.IsMatch(code);
            }
            return valid;
        }

        private bool CheckEAN(string code)
        {
            if (code == null || code.Length < 8) return false;

            const string pattern = @"^([0-9]+)$";
            var check = new Regex(pattern, RegexOptions.IgnoreCase);

            var valid = false;
            if (!string.IsNullOrEmpty(code))
            {
                valid = check.IsMatch(code);
            }
            return valid;
        }

        private void textBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter || (e.KeyValue < 67 && e.KeyValue > 65 && oldKeyValue == 17))
            {
                if (tbMarkCode.Text.Trim().Length == 68)
                {
                    if (CheckMarkCode(tbMarkCode.Text.Trim()))
                    {
                        tbEAN.Focus();
                        tbEAN.SelectAll();
                    }
                    else
                    {
                        MessageBox.Show(this, "Неверный код марки", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        tbMarkCode.SelectAll();
                    }
                }
            }
            oldKeyValue = e.KeyValue;
        }

        private void tbEAN_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter || (e.KeyValue < 67 && e.KeyValue > 65 && oldKeyValue == 17))
            {
                if (string.IsNullOrEmpty(tbEAN.Text.Trim()) || CheckEAN(tbEAN.Text.Trim()))
                {
                    if (!CheckMarkCode(tbMarkCode.Text.Trim()))
                    {
                        MessageBox.Show(this, "Неверный код марки", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        tbMarkCode.SelectAll();
                        tbMarkCode.Focus();
                        return;
                    }

                    //Пишем в файл
                    streamWriter.WriteLine("{0} {1}", tbMarkCode.Text.Trim(), tbEAN.Text.Trim());
                    streamWriter.Flush();

                    tbMarkCode.Text = string.Empty;
                    tbEAN.Text = string.Empty;
                    tbMarkCode.Focus();
                }
                else
                {
                    MessageBox.Show(this, "Неверный ШК", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    tbEAN.SelectAll();
                }
            }
            oldKeyValue = e.KeyValue;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            streamWriter.Close();
            fileStream.Close();
            //Освобождаем сканер
            UtilsScannerBarcode.CloseScanner();
        }

        private void cbComPort_CheckedChanged(object sender, EventArgs e)
        {
            if (cbComPort.Checked)
            {
                cbComPortName.Enabled = true;

                if (!UtilsScannerBarcode.ConnectScanner(port_DataReceived, cbComPortName.SelectedItem.ToString()))
                {
                    MessageBox.Show(this, "Ошибка открытия COM-порта", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            else
            {
                cbComPortName.Enabled = false;
                //Освобождаем сканер
                UtilsScannerBarcode.CloseScanner();
            }

        }

        private void cbComPortName_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!cbComPort.Checked) return;
            //Освобождаем сканер
            UtilsScannerBarcode.CloseScanner();

            if (!UtilsScannerBarcode.ConnectScanner(port_DataReceived, cbComPortName.SelectedItem.ToString()))
            {
                MessageBox.Show(this, "Ошибка открытия COM-порта", "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
    }
}
